@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -32,6 +32,14 @@ import re
 import time
 import md5
 
+def statement_name(name):
+    """
+    Reports the name of this statement to systems like window auto.
+    """
+
+    for i in renpy.config.statement_callbacks:
+        i(name)
+
 def next_node(n):
     """
     Indicates the next node that should be executed. When a statement
@@ -63,6 +71,77 @@ class ParameterInfo(object):
         # any. None if no such variable exists.
         self.extrakw = extrakw
 
+
+    def apply(self, args, kwargs):
+        """
+        Applies `args` and `kwargs` to these parameters. Returns
+        a dictionary that can be used to update an enclosing
+        scope.
+        """
+
+        values = { }
+        rv = { }
+
+        if args is None:
+            args = ()
+
+        if kwargs is None:
+            kwargs = { }
+
+
+        for name, value in zip(self.positional, args):
+            if name in values:
+                raise Exception("Parameter %s has two values." % name)
+
+            values[name] = value
+
+        extrapos = tuple(args[len(self.positional):])
+
+        for name, value in kwargs.iteritems():
+            if name in values:
+                raise Exception("Parameter %s has two values." % name)
+
+            values[name] = value
+
+        for name, default in self.parameters:
+
+            if name not in values:
+                if default is None:
+                    raise Exception("Required parameter %s has no value." % name)
+                else:
+                    rv[name] = renpy.python.py_eval(default)
+
+            else:
+                rv[name] = values[name]
+                del values[name]
+
+        # Now, values has the left-over keyword arguments, and extrapos
+        # has the left-over positional arguments.
+
+        if self.extrapos:
+            rv[self.extrapos] = extrapos
+        elif extrapos:
+            raise Exception("Too many arguments in call (expected %d, got %d)." % (len(self.positional), len(args)))
+
+        if self.extrakw:
+            rv[self.extrakw] = values
+        else:
+            if values:
+                raise Exception("Unknown keyword arguments: %s" % ( ", ".join(values.keys())))
+
+        return rv
+
+def apply_arguments(parameters, args, kwargs):
+
+    if parameters is None:
+        if args or kwargs:
+            raise Exception("Arguments supplied, but parameter list not present")
+        else:
+            return { }
+
+    return parameters.apply(args, kwargs)
+
+
 class ArgumentInfo(object):
 
     def __init__(self, arguments, extrapos, extrakw):
@@ -79,6 +158,29 @@ class ArgumentInfo(object):
         # to be supplied to this function.
         self.extrakw = extrakw
 
+    def evaluate(self, scope=None):
+        """
+        Evaluates the arguments, returning a list of arguments and a
+        dictionary of keyword arguments.
+        """
+
+        args = [ ]
+        kwargs = renpy.python.RevertableDict()
+
+        for k, v in self.arguments:
+            if k is not None:
+                kwargs[k] = renpy.python.py_eval(v, locals=scope)
+            else:
+                args.append(renpy.python.py_eval(v, locals=scope))
+
+        if self.extrapos is not None:
+            args.extend(renpy.python.py_eval(self.extrapos, locals=scope))
+
+        if self.extrakw is not None:
+            kwargs.update(renpy.python.py_eval(self.extrakw, locals=scope))
+
+        return tuple(args), kwargs
+
 
 def __newobj__(cls, *args):
     return cls.__new__(cls, *args)
@@ -419,10 +521,11 @@ class Say(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("say")
 
         try:
 
-            renpy.exports.say_attributes = self.attributes
+            renpy.game.context().say_attributes = self.attributes
 
             if self.who is not None:
                 if self.who_fast:
@@ -455,16 +558,16 @@ class Say(Node):
             renpy.exports.say(who, what, interact=self.interact)
 
         finally:
-            renpy.exports.say_attributes = None
+            renpy.game.context().say_attributes = None
 
 
     def predict(self):
 
-        old_attributes = renpy.exports.say_attributes
+        old_attributes = renpy.game.context().say_attributes
 
         try:
 
-            renpy.exports.say_attributes = self.attributes
+            renpy.game.context().say_attributes = self.attributes
 
             if self.who is not None:
                 if self.who_fast:
@@ -486,7 +589,7 @@ class Say(Node):
             renpy.exports.predict_say(who, what)
 
         finally:
-            renpy.exports.say_attributes = old_attributes
+            renpy.game.context().say_attributes = old_attributes
 
         return [ self.next ]
 
@@ -541,80 +644,11 @@ class Init(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("init")
 
     def restructure(self, callback):
         callback(self.block)
 
-def apply_arguments(params, args, kwargs):
-    """
-    Applies arguments to parameters to update scope.
-
-    `scope`
-        A dict.
-
-    `params`
-        The parameters object.
-
-    `args`, `kwargs`
-        Positional and keyword arguments.
-    """
-
-    values = { }
-    rv = { }
-
-    if args is None:
-        args = ()
-
-    if kwargs is None:
-        kwargs = { }
-
-    if params is None:
-        if args or kwargs:
-            raise Exception("Arguments supplied, but parameter list not present")
-        else:
-            return rv
-
-    for name, value in zip(params.positional, args):
-        if name in values:
-            raise Exception("Parameter %s has two values." % name)
-
-        values[name] = value
-
-    extrapos = tuple(args[len(params.positional):])
-
-    for name, value in kwargs.iteritems():
-        if name in values:
-            raise Exception("Parameter %s has two values." % name)
-
-        values[name] = value
-
-    for name, default in params.parameters:
-
-        if name not in values:
-            if default is None:
-                raise Exception("Required parameter %s has no value." % name)
-            else:
-                rv[name] = renpy.python.py_eval(default)
-
-        else:
-            rv[name] = values[name]
-            del values[name]
-
-    # Now, values has the left-over keyword arguments, and extrapos
-    # has the left-over positional arguments.
-
-    if params.extrapos:
-        rv[params.extrapos] = extrapos
-    elif extrapos:
-        raise Exception("Too many arguments in call (expected %d, got %d)." % (len(params.positional), len(args)))
-
-    if params.extrakw:
-        rv[params.extrakw] = values
-    else:
-        if values:
-            raise Exception("Unknown keyword arguments: %s" % ( ", ".join(values.keys())))
-
-    return rv
 
 class Label(Node):
 
@@ -662,6 +696,7 @@ class Label(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("label")
 
         renpy.game.context().mark_seen()
 
@@ -715,6 +750,7 @@ class Python(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("python")
 
         try:
             renpy.python.py_exec_bytecode(self.code.bytecode, self.hide, store=self.store)
@@ -760,6 +796,7 @@ class EarlyPython(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("python early")
 
     def early_execute(self):
         renpy.python.create_store(self.store)
@@ -803,6 +840,7 @@ class Image(Node):
         # accessing self.atl, as self.atl may not always exist.
 
         next_node(self.next)
+        statement_name("image")
 
         if self.code is not None:
             img = renpy.python.py_eval_bytecode(self.code.bytecode)
@@ -843,6 +881,7 @@ class Transform(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("transform")
 
         parameters = getattr(self, "parameters", None)
 
@@ -861,14 +900,15 @@ def predict_imspec(imspec, scene=False, atl=None):
     """
 
     if len(imspec) == 7:
-        name, expression, tag, _at_list, layer, _zorder, _behind = imspec
+        name, expression, tag, at_expr_list, layer, _zorder, _behind = imspec
 
     elif len(imspec) == 6:
-        name, expression, tag, _at_list, layer, _zorder = imspec
+        name, expression, tag, at_expr_list, layer, _zorder = imspec
 
     elif len(imspec) == 3:
-        name, _at_list, layer = imspec
-
+        name, at_expr_list, layer = imspec
+        tag = None
+        expression = None
 
     if expression:
         try:
@@ -876,30 +916,27 @@ def predict_imspec(imspec, scene=False, atl=None):
             img = renpy.easy.displayable(img)
         except:
             return
-
     else:
-        img = renpy.display.image.images.get(name, None)
-        if img is None:
-            return
+        img = None
 
-    full_name = name
-    if tag:
-        full_name = (tag,) + full_name[1:]
+    at_list = [ ]
+    for i in at_expr_list:
+        try:
+            at_list.append(renpy.python.py_eval(i))
+        except:
+            pass
+
+
+    if atl is not None:
+        try:
+            at_list.append(renpy.display.motion.ATLTransform(atl))
+        except:
+            pass
 
     if scene:
         renpy.game.context().images.predict_scene(layer)
 
-    renpy.game.context().images.predict_show(tag or name, layer)
-
-    if atl is not None:
-        try:
-            img = renpy.display.motion.ATLTransform(atl, child=img)
-        except:
-            import traceback
-            traceback.print_exc()
-
-    renpy.display.predict.displayable(img)
-
+    renpy.exports.predict_show(name, layer, what=img, tag=tag)
 
 
 def show_imspec(imspec, atl=None):
@@ -961,6 +998,7 @@ class Show(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("show")
 
         show_imspec(self.imspec, atl=getattr(self, "atl", None))
 
@@ -969,6 +1007,40 @@ class Show(Node):
         return [ self.next ]
 
 
+class ShowLayer(Node):
+
+    __slots__ = [
+        'layer',
+        'at_list',
+        'atl',
+        ]
+
+    def __init__(self, loc, layer, at_list, atl):
+        super(ShowLayer, self).__init__(loc)
+
+        self.layer = layer
+        self.at_list = at_list
+        self.atl = atl
+
+    def diff_info(self):
+        return (ShowLayer, self.layer)
+
+    def execute(self):
+        next_node(self.next)
+        statement_name("show layer")
+
+        at_list = [ renpy.python.py_eval(i) for i in self.at_list ]
+
+        if self.atl is not None:
+            atl = renpy.display.motion.ATLTransform(self.atl)
+            at_list.append(atl)
+
+        renpy.exports.layer_at_list(at_list, layer=self.layer)
+
+    def predict(self):
+        return [ self.next ]
+
+
 class Scene(Node):
 
     __slots__ = [
@@ -1002,6 +1074,7 @@ class Scene(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("scene")
 
         renpy.config.scene(self.layer)
 
@@ -1061,6 +1134,7 @@ class Hide(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("hide")
 
         if len(self.imspec) == 3:
             name, _at_list, layer = self.imspec
@@ -1101,6 +1175,7 @@ class With(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("with")
 
         trans = renpy.python.py_eval(self.expr)
 
@@ -1150,6 +1225,8 @@ class Call(Node):
 
     def execute(self):
 
+        statement_name("call")
+
         label = self.label
         if self.expression:
             label = renpy.python.py_eval(label)
@@ -1159,34 +1236,8 @@ class Call(Node):
         renpy.game.context().abnormal = True
 
         if self.arguments:
-
-            args = [ ]
-            kwargs = renpy.python.RevertableDict()
-
-            for name, expr in self.arguments.arguments:
-
-                value = renpy.python.py_eval(expr)
-
-                if name is None:
-                    args.append(value)
-                else:
-                    if name in kwargs:
-                        raise Exception("The argument named %s appears twice." % name)
-
-                    kwargs[name] = value
-
-            if self.arguments.extrapos:
-                args.extend(renpy.python.py_eval(self.arguments.extrapos))
-
-            if self.arguments.extrakw:
-                for name, value in renpy.python.py_eval(self.arguments.extrakw).iteritems():
-                    if name in kwargs:
-                        raise Exception("The argument named %s appears twice." % name)
-
-                    kwargs[name] = value
-
-
-            renpy.store._args = tuple(args)
+            args, kwargs = self.arguments.evaluate()
+            renpy.store._args = args
             renpy.store._kwargs = kwargs
 
     def predict(self):
@@ -1223,6 +1274,8 @@ class Return(Node):
 
     def execute(self):
 
+        statement_name("return")
+
         if self.expression:
             renpy.store._return = renpy.python.py_eval(self.expression)
         else:
@@ -1284,6 +1337,7 @@ class Menu(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("menu")
 
         choices = [ ]
         narration = [ ]
@@ -1369,6 +1423,8 @@ class Jump(Node):
 
     def execute(self):
 
+        statement_name("jump")
+
         target = self.target
         if self.expression:
             target = renpy.python.py_eval(target)
@@ -1405,6 +1461,7 @@ class Pass(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("pass")
 
 
 class While(Node):
@@ -1433,6 +1490,7 @@ class While(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("while")
 
         if renpy.python.py_eval(self.condition):
             next_node(self.block[0])
@@ -1481,6 +1539,7 @@ class If(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("if")
 
         for condition, block in self.entries:
             if renpy.python.py_eval(condition):
@@ -1529,11 +1588,17 @@ class UserStatement(Node):
 
     def execute(self):
         next_node(self.get_next())
+        statement_name(self.get_name())
 
         self.call("execute")
 
     def predict(self):
-        self.call("predict")
+        predictions = self.call("predict")
+
+        if predictions is not None:
+            for i in predictions:
+                renpy.easy.predict(i)
+
         return [ self.get_next() ]
 
     def call(self, method, *args, **kwargs):
@@ -1545,6 +1610,14 @@ class UserStatement(Node):
 
         renpy.statements.call(method, parsed, *args, **kwargs)
 
+    def get_name(self):
+        parsed = self.parsed
+        if parsed is None:
+            parsed = renpy.statements.parse(self, self.line, self.block)
+            self.parsed = parsed
+
+        return renpy.statements.get_name(parsed)
+
     def get_next(self):
         rv = self.call("next")
         if rv is not None:
@@ -1588,6 +1661,7 @@ class Define(Node):
     def execute(self):
 
         next_node(self.next)
+        statement_name("define")
 
         value = renpy.python.py_eval_bytecode(self.code.bytecode)
         renpy.dump.definitions.append((self.varname, self.filename, self.linenumber))
@@ -1617,6 +1691,8 @@ class Screen(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("screen")
+
         self.screen.define()
         renpy.dump.screens.append((self.screen.name, self.filename, self.linenumber))
 
@@ -1661,11 +1737,14 @@ class Translate(Node):
 
     def execute(self):
 
+        statement_name("translate")
+
         if self.language is not None:
             next_node(self.next)
             raise Exception("Translation nodes cannot be run directly.")
 
         next_node(renpy.game.script.translator.lookup_translate(self.identifier))
+
         renpy.game.context().translate_identifier = self.identifier
 
     def predict(self):
@@ -1698,6 +1777,8 @@ class EndTranslate(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("end translate")
+
         renpy.game.context().translate_identifier = None
 
 
@@ -1723,9 +1804,16 @@ class TranslateString(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("translate string")
+
         renpy.translation.add_string_translation(self.language, self.old, self.new)
 
 class TranslatePython(Node):
+    """
+    Runs python code when changing the language.
+
+    This is no longer generated, but is still run when encountered.
+    """
 
     __slots__ = [
         'language',
@@ -1750,7 +1838,115 @@ class TranslatePython(Node):
 
     def execute(self):
         next_node(self.next)
+        statement_name("translate_python")
 
     # def early_execute(self):
     #    renpy.python.create_store(self.store)
     #    renpy.python.py_exec_bytecode(self.code.bytecode, self.hide, store=self.store)
+
+
+class TranslateBlock(Node):
+    """
+    Runs a block of code when changing the language.
+    """
+
+    __slots__ = [
+        'block',
+        'language',
+        ]
+
+    def __init__(self, loc, language, block):
+        super(TranslateBlock, self).__init__(loc)
+
+        self.language = language
+        self.block = block
+
+    def get_children(self):
+        return self.block
+
+    # We handle chaining specially. We want to chain together the nodes in
+    # the block, but we want that chain to end in None, and we also want
+    # this node to just continue on to the next node in normal execution.
+    def chain(self, next): #@ReservedAssignment
+        self.next = next
+        chain_block(self.block, None)
+
+    def execute(self):
+        next_node(self.next)
+        statement_name("translate_block")
+
+    def restructure(self, callback):
+        callback(self.block)
+
+class Style(Node):
+
+    __slots__ = [
+        'style_name',
+        'parent',
+        'properties',
+        'clear',
+        'take',
+        'delattr',
+        'variant',
+    ]
+
+    def __init__(self, loc, name):
+        """
+        `name`
+            The name of the style to define.
+        """
+
+        super(Style, self).__init__(loc)
+
+        self.style_name = name
+
+        # The parent of this style.
+        self.parent = None
+
+        # Properties.
+        self.properties = { }
+
+        # Should we clear the style?
+        self.clear = False
+
+        # Should we take properties from another style?
+        self.take = None
+
+        # A list of attributes we should delete from this style.
+        self.delattr = [ ]
+
+        # If not none, an expression for the variant.
+        self.variant = None
+
+    def diff_info(self):
+        return (Style, self.style_name)
+
+    def execute(self):
+        next_node(self.next)
+        statement_name("style")
+
+        if self.variant is not None:
+            variant = renpy.python.py_eval(self.variant)
+            if not renpy.exports.variant(variant):
+                return
+
+        s = renpy.style.get_or_create_style(self.style_name) # @UndefinedVariable
+
+        if self.clear:
+            s.clear()
+
+        if self.parent is not None:
+            s.set_parent(self.parent)
+
+        if self.take is not None:
+            s.take(self.take)
+
+        for i in self.delattr:
+            s.delattr(i)
+
+        if self.properties:
+            properties = { }
+            for name, expr in self.properties.items():
+                properties[name] = renpy.python.py_eval(expr)
+
+            s.add_properties(properties)
