@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -21,11 +21,11 @@
 
 import os.path
 import sys
-import cStringIO
-import platform
-import traceback
+import subprocess
 import io
 
+import renpy.error
+
 FSENCODING = sys.getfilesystemencoding() or "utf-8"
 
 # Extra things used for distribution.
@@ -221,6 +221,7 @@ def bootstrap(renpy_base):
     if renpy.macintosh:
         os.startfile = mac_start
 
+
     # Check that we have installed pygame properly. This also deals with
     # weird cases on Windows and Linux where we can't import modules. (On
     # windows ";" is a directory separator in PATH, so if it's in a parent
@@ -233,7 +234,24 @@ def bootstrap(renpy_base):
 Could not import pygame. Please ensure that this program has been built
 and unpacked properly. Also, make sure that the directories containing
 this program do not contain : or ; in their names.
-"""
+
+You may be using a system install of python. Please run {0}.sh,
+{0}.exe, or {0}.app instead.
+""".format(name)
+
+        raise
+
+    # Ditto for the Ren'Py module.
+    try:
+        import _renpy; _renpy
+    except:
+        print >>sys.stderr, """\
+Could not import _renpy. Please ensure that this program has been built
+and unpacked properly.
+
+You may be using a system install of python. Please run {0}.sh,
+{0}.exe, or {0}.app instead.
+""".format(name)
         raise
 
     # Load up all of Ren'Py, in the right order.
@@ -243,10 +261,12 @@ this program do not contain : or ; in their names.
 
     renpy.loader.init_importer()
 
-    keep_running = True
+    exit_status = None
 
     try:
-        while keep_running:
+        while exit_status is None:
+            exit_status = 1
+
             try:
                 renpy.game.args = args
                 renpy.config.renpy_base = renpy_base
@@ -263,33 +283,33 @@ this program do not contain : or ; in their names.
                     os.makedirs(renpy.config.logdir, 0777)
 
                 renpy.main.main()
-                keep_running = False
+
+                exit_status = 0
 
             except KeyboardInterrupt:
-                traceback.print_exc()
-                break
+                raise
 
             except renpy.game.UtterRestartException:
 
-                if renpy.display.draw:
-                    renpy.display.draw.deinit()
-                    renpy.display.draw.quit()
-
                 # On an UtterRestart, reload Ren'Py.
                 renpy.reload_all()
-                continue
 
-            except renpy.game.QuitException:
-                keep_running = False
+                exit_status = None
+
+            except renpy.game.QuitException as e:
+                exit_status = e.status
+
+                if e.relaunch:
+                    subprocess.Popen([sys.executable, "-EO"] + sys.argv)
 
             except renpy.game.ParseErrorException:
-                keep_running = False
+                pass
 
             except Exception, e:
-                report_exception(e)
-                keep_running = False
+                renpy.error.report_exception(e)
+                pass
 
-        sys.exit(0)
+        sys.exit(exit_status)
 
     finally:
 
@@ -303,219 +323,4 @@ this program do not contain : or ; in their names.
 
         # Prevent subprocess from throwing errors while trying to run it's
         # __del__ method during shutdown.
-        import subprocess # W0403
-        subprocess.Popen.__del__ = popen_del # E1101
-
-def report_line(out, filename, line, what):
-    out.write('  File "%s", line %d, in %s\n' % (filename, line, what))
-    try:
-        fn = renpy.parser.unelide_filename(filename)
-        f = file(fn, "rb")
-        lines = f.read().decode("utf-8").replace("\r", "").split("\n")
-        out.write("    " + lines[line - 1].encode("utf-8") + "\n")
-    except:
-        pass
-
-
-def write_utf8_traceback_list(out, l):
-    """
-    Given the traceback list l, writes it to out as utf-8.
-    """
-
-    ul = [ ]
-
-    for filename, line, what, text in l:
-
-        # Filename is either unicode or an fsecoded string.
-        if not isinstance(filename, unicode):
-            filename = unicode(filename, FSENCODING, "replace")
-
-        # Line is a number.
-
-        # Assume what is in a unicode encoding, since it is either python,
-        # or comes from inside Ren'Py.
-
-        if isinstance(text, str):
-            text = text.decode("utf-8", "replace")
-
-        ul.append((filename, line, what, text))
-
-    for t in traceback.format_list(ul):
-        out.write(t.encode("utf-8", "replace"))
-
-
-def script_level_traceback(out, tb):
-    """
-    Writes a script-level traceback to out, based on the traceback
-    object tb.
-    """
-
-    tbl = [ ]
-
-    while tb:
-        f = tb.tb_frame
-        line = tb.tb_lineno
-        co = f.f_code
-        filename = co.co_filename
-
-        if filename.endswith(".rpy") and not filename.replace("\\", "/").startswith("common/"):
-            tbl.append((filename, line, "python", None))
-
-        elif 'self' in f.f_locals:
-            obj = f.f_locals['self']
-
-            import renpy
-
-            if isinstance(obj, renpy.execution.Context):
-                tbl.extend(obj.report_tb(out))
-
-        tb = tb.tb_next
-
-    write_utf8_traceback_list(out, tbl)
-
-def open_error_file(fn, mode):
-    """
-    Opens an error/log/file. Returns the open file, and the filename that
-    was opened.
-    """
-
-    try:
-        f = file(os.path.join(renpy.config.logdir, fn), mode)
-        return f, fn
-    except:
-        pass
-
-    try:
-        f = file(fn, mode)
-        return f, fn
-    except:
-        pass
-
-    import tempfile
-
-    fn = os.path.join(tempfile.gettempdir(), "renpy-" + fn)
-    return file(fn, mode), fn
-
-def report_exception(e, editor=True):
-    """
-    Reports an exception by writing it to standard error and
-    traceback.txt. If `editor` is True, opens the traceback
-    up in a text editor.
-
-    Returns a two-unicode tuple, with the first item being
-    a simple message, and the second being a full traceback.
-    """
-
-    import codecs
-
-    type, _value, tb = sys.exc_info() #@ReservedAssignment
-
-    def safe_utf8(e):
-        try:
-            m = unicode(e)
-        except:
-            m = str(e)
-
-        if isinstance(m, unicode):
-            return m.encode("utf-8", "replace")
-        else:
-            return m
-
-    # Return values - which can be displayed to the user.
-    simple = cStringIO.StringIO()
-    full = cStringIO.StringIO()
-
-    print >>simple, renpy.game.exception_info
-    script_level_traceback(simple, tb)
-    print >>simple, type.__name__ + ":",
-    print >>simple, safe_utf8(e)
-
-    print >>full, "Full traceback:"
-    tbl = traceback.extract_tb(tb)
-    write_utf8_traceback_list(full, tbl)
-    print >>full, type.__name__ + ":",
-    print >>full, safe_utf8(e)
-
-    # Write to stdout/stderr.
-    sys.stdout.write("\n")
-    sys.stdout.write(full.getvalue())
-    sys.stdout.write("\n")
-    sys.stdout.write(simple.getvalue())
-
-    print >>full
-    try:
-        print >>full, platform.platform()
-        print >>full, renpy.version
-        print >>full, renpy.config.name + " " + renpy.config.version
-    except:
-        pass
-
-
-    simple = simple.getvalue()
-    full = full.getvalue()
-
-    # Inside of the file, which may not be openable.
-    try:
-
-        f, traceback_fn = open_error_file("traceback.txt", "w")
-
-        f.write(codecs.BOM_UTF8)
-
-        print >>f, "I'm sorry, but an uncaught exception occurred."
-        print >>f
-
-        f.write(simple)
-
-        print >>f
-        print >>f, "-- Full Traceback ------------------------------------------------------------"
-        print >>f
-
-        f.write(full)
-        f.close()
-
-        try:
-            if editor and renpy.game.args.command == "run": #@UndefinedVariable
-                renpy.exports.launch_editor([ traceback_fn ], 1, transient=1)
-        except:
-            pass
-
-    except:
-        pass
-
-    try:
-        renpy.display.log.exception() #@UndefinedVariable
-    except:
-        pass
-
-    return simple.decode("utf-8", "replace"), full.decode("utf-8", "replace"), traceback_fn
-
-
-old_memory = { }
-
-def memory_profile():
-    """
-    Calling this function displays the change in the number of instances of
-    each type of object.
-    """
-
-    print "- Memory Profile ---------------------------------------------------"
-
-    import gc
-    gc.collect()
-
-    objs = gc.get_objects()
-
-    c = { } # count
-
-    for i in objs:
-        t = type(i)
-        c[t] = c.get(t, 0) + 1
-
-    results = [ (count, ty) for ty, count in c.iteritems() ]
-    results.sort()
-
-    for count, ty in results:
-        diff = count - old_memory.get(ty, 0)
-        old_memory[ty] = count
-        if diff:
-            print diff, ty
+        subprocess.Popen.__del__ = popen_del
