@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -20,6 +20,10 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 import renpy.display
+import os
+import time
+
+PROFILE = ("RENPY_PROFILE_SCREENS" in os.environ)
 
 class Screen(renpy.object.Object):
     """
@@ -45,9 +49,16 @@ class Screen(renpy.object.Object):
 
         screens[name[0], variant] = self
 
-        # The function that is called to display this screen.
+        # A function that can be called to display the screen.
         self.function = function
 
+        # If this is a SL2 screen, the SLScreen node at the root of this
+        # screen.
+        if isinstance(function, renpy.sl2.slast.SLScreen): # @UndefinedVariable
+            self.ast = function
+        else:
+            self.ast = None
+
         # Expression: Are we modal? (A modal screen ignores screens under it.)
         self.modal = modal
 
@@ -73,7 +84,7 @@ class ScreenDisplayable(renpy.display.layout.Container):
     class is responsible for managing the display of a screen.
     """
 
-    nosave = [ 'screen', 'child', 'transforms', 'widgets', 'old_widgets', 'old_transforms' ]
+    nosave = [ 'screen', 'child', 'transforms', 'widgets', 'old_widgets', 'old_transforms', "cache" ]
 
     restarting = False
 
@@ -84,6 +95,7 @@ class ScreenDisplayable(renpy.display.layout.Container):
         self.widgets = { }
         self.old_widgets = None
         self.old_transforms = None
+        self.cache = { }
 
     def __init__(self, screen, tag, layer, widget_properties={}, scope={}, **properties):
 
@@ -115,6 +127,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
         # A map from name to the widget with that name.
         self.widgets = { }
 
+        # The persistent cache.
+        self.cache = { }
+
         if tag and layer:
             old_screen = get_screen(tag, layer)
         else:
@@ -155,8 +170,8 @@ class ScreenDisplayable(renpy.display.layout.Container):
         self.modal = renpy.python.py_eval(self.screen.modal, locals=self.scope)
         self.zorder = renpy.python.py_eval(self.screen.zorder, locals=self.scope)
 
-    def __repr__(self):
-        return "<ScreenDisplayable: %r>" % (self.screen_name,)
+    def __unicode__(self):
+        return "Screen {}".format(" ".join(self.screen_name))
 
     def visit(self):
         return [ self.child ]
@@ -233,6 +248,10 @@ class ScreenDisplayable(renpy.display.layout.Container):
 
     def update(self):
 
+        if self.screen is None:
+            self.child = renpy.display.layout.Null()
+            return { }
+
         # If we're restarting, do not update - the update can use variables
         # that are no longer in scope.
         if self.restarting:
@@ -241,6 +260,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
 
             return self.widgets
 
+        if PROFILE:
+            start = time.time()
+
         # Update _current_screen
         global _current_screen
         old_screen = _current_screen
@@ -285,6 +307,10 @@ class ScreenDisplayable(renpy.display.layout.Container):
 
             self.current_transform_event = None
 
+        if PROFILE:
+            end = time.time()
+            print "screen {} took {:.3f}ms".format(" ".join(self.screen_name).encode("utf-8"), 1000.0 * (end - start))
+
         return self.widgets
 
     def render(self, w, h, st, at):
@@ -576,7 +602,7 @@ def use_screen(_screen_name, *_args, **kwargs):
     screen = get_screen_variant(name[0])
 
     if screen is None:
-        raise Exception("Screen %r is not known." % name)
+        raise Exception("Screen %r is not known." % (name,))
 
     old_transfers = _current_screen.old_transfers
     _current_screen.old_transfers = True
