@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -701,7 +701,7 @@ class SWDraw(object):
         renpy.display.interface.post_init()
 
         if self.display_info is None:
-            self.display_info = pygame.display.Info()
+            self.display_info = renpy.display.get_info()
 
         # The scale factor we use for this display.
         self.scale_factor = 1.0
@@ -752,7 +752,9 @@ class SWDraw(object):
 
         if ((old_screen is not None) and
             (old_screen.get_size() == (scaled_width, scaled_height)) and
-            (old_screen.get_flags() & pygame.FULLSCREEN == fsflag)):
+            (old_screen.get_flags() & pygame.FULLSCREEN == fsflag) and
+            not (old_screen.get_flags() & pygame.OPENGL)
+            ):
 
             self.screen = old_screen
 
@@ -886,6 +888,13 @@ class SWDraw(object):
 
         return x, y
 
+    def set_mouse_pos(self, x, y):
+
+        x *= self.scale_factor
+        y *= self.scale_factor
+
+        return pygame.mouse.set_pos([x, y])
+
 
     def screenshot(self, surftree, fullscreen_video):
         """
