@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -56,7 +56,7 @@ class Context(renpy.object.Object):
     does participates in rollback.
     """
 
-    __version__ = 10
+    __version__ = 11
 
     def after_upgrade(self, version):
         if version < 1:
@@ -93,6 +93,9 @@ class Context(renpy.object.Object):
         if version < 10:
             self.exception_handler = None
 
+        if version < 11:
+            self.say_attributes = None
+
     def __init__(self, rollback, context=None, clear=False):
         """
         `clear`
@@ -145,6 +148,9 @@ class Context(renpy.object.Object):
         # code. If None, a default handler is used. This is reset when run is called.
         self.exception_handler = None
 
+        # The attributes that are used by the current say statement.
+        self.say_attributes = None
+
         if context:
             oldsl = context.scene_lists
             self.runtime = context.runtime
@@ -235,7 +241,7 @@ class Context(renpy.object.Object):
 
         self.current = node_name
 
-    def report_tb(self, out):
+    def report_traceback(self, name):
 
         rv = [ ]
 
@@ -298,7 +304,7 @@ class Context(renpy.object.Object):
                     self.translate_interaction = None
 
                     exc_info = sys.exc_info()
-                    short, full, traceback_fn = renpy.bootstrap.report_exception(e, editor=False)
+                    short, full, traceback_fn = renpy.error.report_exception(e, editor=False)
 
                     try:
                         if self.exception_handler is not None:
