@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -35,7 +35,7 @@ from renpy.text.font import register_sfont, register_mudgefont, register_bmfont
 from renpy.text.text import language_tailor
 from renpy.display.behavior import Keymap
 from renpy.display.behavior import run as run_action, run_unhovered, run_periodic
-from renpy.display.behavior import map_event
+from renpy.display.behavior import map_event, queue_event, clear_keymap_cache
 
 from renpy.display.minigame import Minigame
 from renpy.display.screen import define_screen, show_screen, hide_screen, use_screen, current_screen, has_screen, get_screen, get_widget
@@ -104,7 +104,7 @@ def public_api():
     register_persistent
     register_statement
     check_text_tags
-
+    map_event, queue_event, clear_keymap_cache
 
 del public_api
 
@@ -188,6 +188,17 @@ def fix_rollback():
     """
     renpy.game.log.fix_rollback()
 
+def retain_after_load():
+    """
+    :doc: retain_after_load
+
+    Causes data modified between the current statement and the statement
+    containing the next checkpoint to be retained when a load occurs.
+    """
+
+    renpy.game.log.retain_after_load()
+
+
 scene_lists = renpy.display.core.scene_lists
 
 def count_displayables_in_layer(layer):
@@ -284,6 +295,63 @@ def showing(name, layer='master'):
 
     return renpy.game.context().images.showing(layer, name)
 
+def predict_show(name, layer='master', what=None, tag=None, at_list=[ ]):
+    """
+    :undocumented:
+
+    Predicts a scene or show statement.
+
+    `name`
+        The name of the image to show, a string.
+
+    `layer`
+        The layer the image is being show non.
+
+    `what`
+        What is being show - if given, overrides `name`.
+
+    `tag`
+        The tag of the thing being shown.
+
+    `at_list`
+        A list of transforms to apply to the displayable.
+    """
+
+    key = tag or name[0]
+
+    if what is None:
+        what = name
+    elif isinstance(what, basestring):
+        what = tuple(what.split())
+
+    if isinstance(what, renpy.display.core.Displayable):
+        base = img = what
+
+    else:
+        if renpy.config.image_attributes:
+
+            new_what = renpy.game.context().images.apply_attributes(layer, key, name)
+            if new_what is not None:
+                what = new_what
+                name = (key,) + new_what[1:]
+
+        base = img = renpy.display.image.ImageReference(what, style='image_placement')
+
+        if not base.find_target():
+            return
+
+    for i in at_list:
+        if isinstance(i, renpy.display.motion.Transform):
+            img = i(child=img)
+        else:
+            img = i(img)
+
+    renpy.game.context().images.predict_show(layer, name, True)
+    renpy.display.predict.displayable(img)
+
+
+
+
 def show(name, at_list=[ ], layer='master', what=None, zorder=0, tag=None, behind=[ ], atl=None, transient=False, munge_name=True):
     """
     :doc: se_images
@@ -448,7 +516,7 @@ def watch(expression, style='default', **properties):
 
     renpy.config.overlay_functions.append(overlay_func)
 
-def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=None): #@ReservedAssignment
+def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=None, pixel_width=None): #@ReservedAssignment
     """
     :doc: input
 
@@ -472,6 +540,10 @@ def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=N
     `length`
         If not None, this must be an integer giving the maximum length
         of the input string.
+
+    `pixel_width`
+        If not None, the input is limited to being this many pixels wide,
+        in the font used by the input to display text.
     """
 
     renpy.exports.mode('input')
@@ -488,7 +560,7 @@ def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=N
 
     if has_screen("input"):
         widget_properties = { }
-        widget_properties["input"] = dict(default=default, length=length, allow=allow, exclude=exclude, editable=not fixed)
+        widget_properties["input"] = dict(default=default, length=length, allow=allow, exclude=exclude, editable=not fixed, pixel_width=pixel_width)
 
         show_screen("input", _transient=True, _widget_properties=widget_properties, prompt=prompt)
 
@@ -713,7 +785,7 @@ def display_menu(items,
             else:
                 item_actions.append((label, action))
 
-            show_screen(screen, items=item_actions, _widget_properties=props, _transient=True, **scope)
+            show_screen(screen, items=item_actions, _widget_properties=props, _transient=True, _layer=renpy.config.choice_layer, **scope)
 
     else:
         renpy.ui.window(style=window_style, focus="menu")
@@ -918,7 +990,7 @@ def imagemap(ground, selected, hotspots, unselected=None, overlays=False,
     return rv
 
 
-def pause(delay=None, music=None, with_none=None, hard=False, checkpoint=True):
+def pause(delay=None, music=None, with_none=None, hard=False, checkpoint=None):
     """
     :doc: other
 
@@ -939,10 +1011,18 @@ def pause(delay=None, music=None, with_none=None, hard=False, checkpoint=True):
         as it's hard to distinguish a hard pause from a crashing game.
 
     `checkpoint`
-        If true, a checkpoint is placed so rollback can occur.
+        If true, a checkpoint will be set, and players will be able to roll
+        back to this statement. If false, no checkpoint will be set. If None,
+        a checkpoint will only be set if display is set.
     """
 
 
+    if checkpoint is None:
+        if delay is not None:
+            checkpoint = False
+        else:
+            checkpoint = True
+
     if renpy.config.skipping == "fast":
         return False
 
@@ -1133,11 +1213,9 @@ def toggle_fullscreen():
 def toggle_music():
     """
     :undocumented:
-    Toggles the playing of music.
+    Does nothing.
     """
 
-    renpy.game.preferences.music = not renpy.game.preferences.music
-
 def has_label(name):
     """
     :doc: label
@@ -1203,7 +1281,17 @@ def utter_restart():
 
     raise renpy.game.UtterRestartException()
 
-def quit(relaunch=False): #@ReservedAssignment
+def reload_script():
+    """
+    :doc: other
+
+    Causes Ren'Py to save the game, reload the script, and then load the
+    save.
+    """
+
+    renpy.game.call_in_new_context("_save_reload_game")
+
+def quit(relaunch=False, status=0): #@ReservedAssignment
     """
     :doc: other
 
@@ -1211,9 +1299,13 @@ def quit(relaunch=False): #@ReservedAssignment
 
     `relaunch`
         If true, Ren'Py will run a second copy of itself before quitting.
+
+    `status`
+        The status code Ren'Py will return to the operating system.
+        Generally, 0 is success, and positive integers are failure.
     """
 
-    raise renpy.game.QuitException(relaunch)
+    raise renpy.game.QuitException(relaunch=relaunch, status=status)
 
 def jump(label):
     """
@@ -1271,15 +1363,6 @@ def version(tuple=False): #@ReservedAssignment
 
     return renpy.version
 
-def module_version():
-    """
-    :undocumented:
-    Returns a number corresponding to the current version of the Ren'Py module,
-    or 0 if the module wasn't loaded.
-    """
-
-    return renpy.display.module.version
-
 def transition(trans, layer=None, always=False, force=False):
     """
     :doc: other
@@ -1384,18 +1467,24 @@ def restart_interaction():
 
 def context():
     """
-    Returns an object that is unique to the current context, that
-    participates in rollback and the like.
+    :doc: context
+
+    Returns an object that is unique to the current context. The object
+    is copied when entering a new context, but changes to the copy do
+    not change the original.
+
+    The object is saved and participates in rollback.
     """
 
     return renpy.game.context().info
 
 def context_nesting_level():
     """
+    :doc: context
+
     Returns the nesting level of the current context. This is 0 for the
-    outermost context (the context that is saved, and in which most of
-    the game runs), and greater than zero when in a menu or other nested
-    context.
+    outermost context (the context that is saved, loaded, and rolled-back),
+    and is non-zero in other contexts, such as menu and replay contexts.
     """
 
     return len(renpy.game.contexts) - 1
@@ -1465,6 +1554,8 @@ def log(msg):
 
 def force_full_redraw():
     """
+    :doc: other
+
     Forces the screen to be redrawn in full. Call this after using pygame
     to redraw the screen directly.
     """
@@ -1596,6 +1687,12 @@ def image_size(im):
     return surf.get_size()
 
 def get_at_list(name, layer='master'):
+    """
+    :undocumented:
+
+    Returns the list of transforms being applied to a layer.
+    """
+
     if isinstance(name, basestring):
         name = tuple(name.split())
 
@@ -1603,9 +1700,21 @@ def get_at_list(name, layer='master'):
 
     return renpy.game.context().scene_lists.at_list[layer].get(tag, None)
 
-def layer_at_list(at_list, layer='master'):
+def show_layer_at(at_list, layer='master'):
+    """
+    :doc: se_images
+
+    The python equivalent of the ``show layer`` `layer` ``at`` `at_list`
+    statement.
+    """
+
+    if not isinstance(at_list, list):
+        at_list = [ at_list ]
+
     renpy.game.context().scene_lists.set_layer_at_list(layer, at_list)
 
+layer_at_list = show_layer_at
+
 def free_memory():
     """
     :doc: other
@@ -1617,15 +1726,32 @@ def free_memory():
     renpy.display.interface.kill_textures_and_surfaces()
 
 def easy_displayable(d, none=False):
+    """
+    :undocumented:
+    """
+
     if none:
         return renpy.easy.displayable(d)
     else:
         return renpy.easy.displayable_or_none(d)
 
 def quit_event():
+    """
+    :doc: other
+
+    Triggers a quit event, as if the player clicked the quit button in the
+    window chrome.
+    """
+
     renpy.game.interface.quit_event()
 
 def iconify():
+    """
+    :doc: other
+
+    Iconifies the game.
+    """
+
     renpy.game.interface.iconify()
 
 # New context stuff.
@@ -1678,6 +1804,20 @@ def end_interaction(value):
     raise renpy.display.core.EndInteraction(value)
 
 def scry():
+    """
+    :doc: other
+
+    Returns the scry object for the current statement.
+
+    The scry object tells Ren'Py about things that must be true in the
+    future of the current statement. Right now, the scry object has one
+    field:
+
+    ``nvl_clear``
+        Is true if an ``nvl clear`` statement will execute before the
+        next interaction.
+    """
+
     name = renpy.game.context().current
     node = renpy.game.script.lookup(name)
     return node.scry()
@@ -1735,6 +1875,8 @@ def load_module(name, **kwargs):
 
 def load_string(s, filename="<string>"):
     """
+    :doc: other
+
     Loads `s` as Ren'Py script that can be called.
 
     Returns the name of the first statement in s.
@@ -1773,6 +1915,16 @@ def load_string(s, filename="<string>"):
         renpy.game.exception_info = old_exception_info
 
 def pop_call():
+    """
+    :doc: other
+
+    Pops the current call from the call stack, without returning to
+    the location.
+
+    This can be used if a label that is called decides not to return
+    to its caller.
+    """
+
     renpy.game.context().pop_dynamic()
     renpy.game.context().lookup_return(pop=True)
 
@@ -1780,13 +1932,15 @@ pop_return = pop_call
 
 def call_stack_depth():
     """
-    Returns the depth of the call stack.
+    :doc: other
+
+    Returns the depth of the call stack of the current context - the number
+    of calls that have run without being returned from or popped from the
+    call stack.
     """
 
     return len(renpy.game.context().return_stack)
 
-
-
 def game_menu(screen=None):
     """
     :undocumented: Probably not what we want in the presence of
@@ -1844,6 +1998,30 @@ def get_placement(d):
 
     return placement(p)
 
+def get_image_bounds(tag, width=None, height=None, layer='master'):
+    """
+    :doc: image_func
+
+    If an image with `tag` exists on `layer`, returns the bounding box of
+    that image. Returns None if the image is not found.
+
+    The bounding box is an (x, y, width, height) tuple. The components of
+    the tuples are expressed in pixels, and may be floating point numbers.
+
+    `width`, `height`
+        The width and height of the area that contains the image. If None,
+        defaults the width and height of the screen, respectively.
+    """
+
+    tag = tag.split()[0]
+
+    if width is None:
+        width = renpy.config.screen_width
+    if height is None:
+        height = renpy.config.screen_height
+
+    return scene_lists().get_image_bounds(layer, tag, width, height)
+
 # User-Defined Displayable stuff.
 
 Render = renpy.display.render.Render
@@ -2050,9 +2228,20 @@ def variant(name):
     by Ren'Py. See :ref:`screen-variants` for more details. This function
     can be used as the condition in a python if statement to set up the
     appropriate styles for the selected screen variant.
+
+    `name` can also be a list of variants, in which case this function
+    returns True if any of the variants is selected.
     """
 
-    return name in renpy.config.variants
+    if isinstance(name, basestring):
+        return name in renpy.config.variants
+    else:
+        for n in name:
+            if n in renpy.config.variants:
+                return True
+
+        return False
+
 
 def vibrate(duration):
     """
@@ -2069,9 +2258,6 @@ def vibrate(duration):
         pass
 
 
-# The attributes that are applied to the current say statement.
-say_attributes = None
-
 def get_say_attributes():
     """
     :doc: other
@@ -2082,7 +2268,8 @@ def get_say_attributes():
     This is only valid when executing or predicting a say statement.
     """
 
-    return say_attributes
+    return renpy.game.context().say_attributes
+
 
 def get_side_image(prefix_tag, image_tag=None, not_showing=True, layer='master'):
     """
@@ -2248,3 +2435,39 @@ def get_mouse_pos():
     currently being touched, x and y are numbers, but not meaningful.
     """
     return renpy.display.draw.get_mouse_pos()
+
+def set_mouse_pos(x, y, duration=0):
+    """
+    :doc: other
+
+    Jump the mouse pointer to the location given by arguments x and y.
+    If the device does not have a mouse pointer, this does nothing.
+
+    `duration`
+        The time it will take to perform the move, in seconds.
+        During this time, the mouse may be unresponsive.
+    """
+
+    renpy.display.interface.set_mouse_pos(x, y, duration)
+
+def set_autoreload(autoreload):
+    """
+    :doc: other
+
+    Sets the autoreload flag, which determines if the game will be
+    automatically reloaded after file changes. Autoreload will not be
+    fully enabled until the game is reloaded with :func:`renpy.utter_restart`.
+    """
+
+    renpy.autoreload = autoreload
+
+def get_autoreload():
+    """
+    :doc: other
+
+    Gets the autoreload flag.
+    """
+
+    return renpy.autoreload
+
+
