@@ -1,5 +1,5 @@
 
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -29,6 +29,7 @@ import os
 
 import renpy.display
 import renpy.ast as ast
+import renpy.sl2
 
 # A list of parse error messages.
 parse_errors = [ ]
@@ -379,20 +380,13 @@ def group_logical_lines(lines):
 
     return gll_core(0, 0)[0]
 
-class Lexer(object):
-    """
-    The lexer that is used to lex script files. This works on the idea
-    that we want to lex each line in a block individually, and use
-    sub-lexers to lex sub-blocks.
-    """
-
-    # A list of keywords which should not be parsed as names, because
-    # there is a huge chance of confusion.
-    #
-    # Note: We need to be careful with what's in here, because thse
-    # are banned in simple_expressions, where we might want to use
-    # some of them.
-    keywords = set([
+# A list of keywords which should not be parsed as names, because
+# there is a huge chance of confusion.
+#
+# Note: We need to be careful with what's in here, because thse
+# are banned in simple_expressions, where we might want to use
+# some of them.
+KEYWORDS = set([
             '$',
             'as',
             'at',
@@ -418,6 +412,42 @@ class Lexer(object):
             'transform',
             ])
 
+OPERATORS = [
+        'or\b',
+        'and\b',
+        'not\b',
+        'in\b',
+        'is\b',
+        '<',
+        '<=',
+        '>',
+        '>=',
+        '<>',
+        '!=',
+        '==',
+        '|',
+        '^',
+        '&',
+        '<<',
+        '>>',
+        '+',
+        '-',
+        '*',
+        '/',
+        '//',
+        '%',
+        '~',
+        '**',
+        ]
+
+operator_regexp = "|".join(re.escape(i) for i in OPERATORS)
+
+class Lexer(object):
+    """
+    The lexer that is used to lex script files. This works on the idea
+    that we want to lex each line in a block individually, and use
+    sub-lexers to lex sub-blocks.
+    """
 
     def __init__(self, block, init=False):
 
@@ -439,6 +469,7 @@ class Lexer(object):
         self.word_cache_newpos = -1
         self.word_cache = ""
 
+
     def advance(self):
         """
         Advances this lexer to the next line in the block. The lexer
@@ -670,7 +701,7 @@ class Lexer(object):
         oldpos = self.pos
         rv = self.word()
 
-        if rv in self.keywords:
+        if rv in KEYWORDS:
             self.pos = oldpos
             return None
 
@@ -831,42 +862,57 @@ class Lexer(object):
         None if it cannot.
         """
 
-        self.skip_whitespace()
-        if self.eol():
-            return None
-
         start = self.pos
 
-        # We start with either a name, a python_string, or parenthesized
-        # python
-        if (not self.python_string() and
-            not self.name() and
-            not self.float() and
-            not self.parenthesised_python()):
+        # Operator.
+        while True:
 
-            return None
-
-        while not self.eol():
-            self.skip_whitespace()
+            while self.match(operator_regexp):
+                pass
 
             if self.eol():
                 break
 
-            # If we see a dot, expect a dotted name.
-            if self.match(r'\.'):
-                n = self.name()
-                if not n:
-                    self.error("expecting name after dot.")
+            # We start with either a name, a python_string, or parenthesized
+            # python
+            if not (self.python_string() or
+                self.name() or
+                self.float() or
+                self.parenthesised_python()):
 
-                continue
+                break
 
-            # Otherwise, try matching parenthesised python.
-            if self.parenthesised_python():
+            while True:
+                self.skip_whitespace()
+
+                if self.eol():
+                    break
+
+                # If we see a dot, expect a dotted name.
+                if self.match(r'\.'):
+                    n = self.name()
+                    if not n:
+                        self.error("expecting name after dot.")
+
+                    continue
+
+                # Otherwise, try matching parenthesised python.
+                if self.parenthesised_python():
+                    continue
+
+                break
+
+            if self.match(operator_regexp):
                 continue
 
             break
 
-        return renpy.ast.PyExpr(self.text[start:self.pos], self.filename, self.number)
+        text = self.text[start:self.pos].strip()
+
+        if not text:
+            return None
+
+        return renpy.ast.PyExpr(self.text[start:self.pos].strip(), self.filename, self.number)
 
     def checkpoint(self):
         """
@@ -925,7 +971,7 @@ class Lexer(object):
 
         pos = self.pos
         self.pos = len(self.text)
-        return self.text[pos:]
+        return renpy.ast.PyExpr(self.text[pos:].strip(), self.filename, self.number)
 
     def python_block(self):
         """
@@ -1589,6 +1635,29 @@ def show_statement(l, loc):
 
     return rv
 
+@statement("show layer")
+def show_layer_statement(l, loc):
+
+    layer = l.require(l.name)
+
+    if l.keyword("at"):
+        at_list = parse_simple_expression_list(l)
+    else:
+        at_list = [ ]
+
+    if l.match(':'):
+        atl = renpy.atl.parse_atl(l.subblock_lexer())
+    else:
+        atl = None
+        l.expect_noblock('show layer statement')
+
+    l.expect_eol()
+    l.advance()
+
+    rv = ast.ShowLayer(loc, layer, at_list, atl)
+
+    return rv
+
 
 @statement("hide")
 def hide_statement(l, loc):
@@ -1760,28 +1829,7 @@ def init_statement(l, loc):
     else:
         priority = 0
 
-    if l.keyword('python'):
-
-        hide = False
-        if l.keyword('hide'):
-
-            hide = True
-
-        if l.keyword('in'):
-            store = "store." + l.require(l.name)
-        else:
-            store = "store"
-
-        l.require(':')
-        l.expect_block('python block')
-
-        python_code = l.python_block()
-
-        l.advance()
-        block = [ ast.Python(loc, python_code, hide, store=store) ]
-
-    else:
-        l.require(':')
+    if l.match(':'):
 
         l.expect_eol()
         l.expect_block('init statement')
@@ -1790,6 +1838,17 @@ def init_statement(l, loc):
 
         l.advance()
 
+    else:
+
+        try:
+            old_init = l.init
+            l.init = True
+
+            block = [ parse_statement(l) ]
+
+        finally:
+            l.init = old_init
+
     return ast.Init(loc, block, priority)
 
 
@@ -1799,6 +1858,7 @@ def screen_statement(l, loc):
     # The guts of screen language parsing is in screenlang.py. It
     # assumes we ate the "screen" keyword before it's called.
     screen = renpy.screenlang.parse_screen(l)
+
     l.advance()
 
     if not screen:
@@ -1811,6 +1871,24 @@ def screen_statement(l, loc):
 
     return rv
 
+
+# @statement("screen2")
+# def screen2_statement(l, loc):
+#
+#     # The guts of screen language parsing is in screenlang.py. It
+#     # assumes we ate the "screen" keyword before it's called.
+#     screen = renpy.sl2.slparser.parse_screen(l, loc)
+#
+#     l.advance()
+#
+#     rv = ast.Screen(loc, screen)
+#
+#     if not l.init:
+#         rv = ast.Init(loc, [ rv ], -500)
+#
+#     return rv
+
+
 def translate_strings(init_loc, language, l):
     l.require(':')
     l.expect_eol()
@@ -1868,17 +1946,6 @@ def translate_strings(init_loc, language, l):
 
     return ast.Init(init_loc, block, 0)
 
-def translate_python(loc, language, l):
-    l.require(':')
-    l.expect_block('python block')
-
-    python_code = l.python_block()
-
-    l.advance()
-
-    return ast.TranslatePython(loc, language, python_code)
-
-
 @statement("translate")
 def translate_statement(l, loc):
 
@@ -1891,8 +1958,27 @@ def translate_statement(l, loc):
 
     if identifier == "strings":
         return translate_strings(loc, language, l)
+
     elif identifier == "python":
-        return translate_python(loc, language, l)
+        try:
+            old_init = l.init
+            l.init = True
+
+            block = [ python_statement(l, loc) ]
+            return [ ast.TranslateBlock(loc, language, block) ]
+        finally:
+            l.init = old_init
+
+    elif identifier == "style":
+        try:
+            old_init = l.init
+            l.init = True
+
+            block = [ style_statement(l, loc) ]
+            return [ ast.TranslateBlock(loc, language, block) ]
+        finally:
+            l.init = old_init
+
 
     l.require(':')
     l.expect_eol()
@@ -1906,6 +1992,96 @@ def translate_statement(l, loc):
     return [ ast.Translate(loc, identifier, language, block), ast.EndTranslate(loc) ]
 
 
+@statement("style")
+def style_statement(l, loc):
+
+    # Parse priority and name.
+    name = l.require(l.word)
+    parent = None
+
+    rv = ast.Style(loc, name)
+
+    # Function that parses a clause. This returns true if a clause has been
+    # parsed, False otherwise.
+    def parse_clause(l):
+
+        if l.keyword("is"):
+            if parent is not None:
+                l.error("parent clause appears twice.")
+
+            rv.parent = l.require(l.word)
+            return True
+
+        if l.keyword("clear"):
+            rv.clear = True
+            return True
+
+        if l.keyword("take"):
+            if rv.take is not None:
+                l.error("take clause appears twice.")
+
+            rv.take = l.require(l.name)
+            return True
+
+        if l.keyword("del"):
+            propname = l.require(l.name)
+
+            if propname not in renpy.style.prefixed_all_properties: # @UndefinedVariable
+                l.error("style property %s is not known." % propname)
+
+            rv.delattr.append(propname)
+            return True
+
+        if l.keyword("variant"):
+            if rv.variant is not None:
+                l.error("variant clause appears twice.")
+
+            rv.variant = l.require(l.simple_expression)
+
+            return True
+
+        propname = l.name()
+
+        if propname is not None:
+            if propname not in renpy.style.prefixed_all_properties: # @UndefinedVariable
+                l.error("style property %s is not known." % propname)
+
+            if propname in rv.properties:
+                l.error("style property %s appears twice." % propname)
+
+            rv.properties[propname] = l.require(l.simple_expression)
+
+            return True
+
+        return False
+
+    while parse_clause(l):
+        pass
+
+    if not l.match(':'):
+        l.expect_noblock("style statement")
+        l.expect_eol()
+    else:
+        l.expect_block("style statement")
+        l.expect_eol()
+
+        ll = l.subblock_lexer()
+
+        while ll.advance():
+
+            while parse_clause(ll):
+                pass
+
+            ll.expect_eol()
+
+    if not l.init:
+        rv = ast.Init(loc, [ rv ], 0)
+
+    l.advance()
+
+    return rv
+
+
 @statement("")
 def say_statement(l, loc):
 
@@ -2062,7 +2238,7 @@ def report_parse_errors():
 
     full_text = ""
 
-    f, error_fn = renpy.bootstrap.open_error_file("errors.txt", "w")
+    f, error_fn = renpy.error.open_error_file("errors.txt", "w")
     f.write(codecs.BOM_UTF8)
 
     print >>f, "I'm sorry, but errors were detected in your script. Please correct the"
