@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -90,6 +90,15 @@ class StoreDict(dict):
         # end of the init phase.
         self.ever_been_changed = set()
 
+    def reset(self):
+        """
+        Called to reset this to its initial conditions.
+        """
+
+        self.old = { }
+        self.ever_been_changed = set()
+        self.clear()
+
     def begin(self):
         """
         Called to mark the start of a rollback period.
@@ -406,19 +415,30 @@ def escape_unicode(s):
 
     return s
 
-def py_compile(source, mode, filename='<none>', lineno=1):
+def py_compile(source, mode, filename='<none>', lineno=1, ast_node=False):
     """
     Compiles the given source code using the supplied codegenerator.
     Lists, List Comprehensions, and Dictionaries are wrapped when
     appropriate.
 
-    @param source: The source code, as a string.
+    `source`
+        The source code, as a either a string, pyexpr, or ast module
+        node.
 
-    @param mode: 'exec' or 'eval'.
+    `mode`
+        One of "exec" or "eval".
 
-    @param filename: The filename that the source code is taken from.
+    `filename`
+        The filename the source comes from. If a pyexpr is given, the
+        filename embedded in the pyexpr is used.
 
-    @param lineno: The line number of the first line of the source code.
+    `lineno`
+        The line number of the first line of source code. If a pyexpr is
+        given, the filename embedded in the pyexpr is used.
+
+    `ast_node`
+        Rather than returning compiled bytecode, returns the AST object
+        that would be used.
     """
 
     if isinstance(source, ast.Module):
@@ -444,6 +464,9 @@ def py_compile(source, mode, filename='<none>', lineno=1):
 
         line_offset = 0
 
+        if ast_node:
+            return tree.body
+
         return compile(tree, filename, mode)
 
     except SyntaxError, e:
@@ -453,6 +476,7 @@ def py_compile(source, mode, filename='<none>', lineno=1):
 
         raise e
 
+
 def py_compile_exec_bytecode(source, **kwargs):
     code = py_compile(source, 'exec', **kwargs)
     return marshal.dumps(code)
@@ -639,7 +663,7 @@ class RevertableSet(sets.Set):
 class RevertableObject(object):
 
     def __new__(cls, *args, **kwargs):
-        self = super(RevertableObject, cls).__new__(cls, *args, **kwargs)
+        self = super(RevertableObject, cls).__new__(cls)
 
         log = renpy.game.log
         if log is not None:
@@ -734,7 +758,7 @@ class Rollback(renpy.object.Object):
     execution of this element.
     """
 
-    __version__ = 2
+    __version__ = 3
 
     def __init__(self):
 
@@ -750,6 +774,9 @@ class Rollback(renpy.object.Object):
         # A map of maps name -> (variable -> value)
         self.stores = { }
 
+        # If true, we retain the data in this rollback when a load occurs.
+        self.retain_after_load = False
+
     def after_upgrade(self, version):
 
         if version < 2:
@@ -763,6 +790,9 @@ class Rollback(renpy.object.Object):
                     k, = i
                     self.stores["store"][k] = deleted
 
+        if version < 3:
+            self.retain_after_load = False
+
 
     def purge_unreachable(self, reachable, wait):
         """
@@ -815,8 +845,8 @@ class Rollback(renpy.object.Object):
 
     def rollback(self):
         """
-        This reverts the game state to the state it was in when this
-        Rollback was first created.
+        Reverts the state of the game to what it was at the start of the
+        previous checkpoint.
         """
 
         for obj, roll in reversed(self.objects):
@@ -835,10 +865,20 @@ class Rollback(renpy.object.Object):
                 else:
                     store[name] = value
 
+        rng.pushback(self.random)
+
+        renpy.game.contexts.pop()
+        renpy.game.contexts.append(self.context)
+
+    def rollback_control(self):
+        """
+        This rolls back only the control information, while leaving
+        the data information intact.
+        """
+
         renpy.game.contexts.pop()
         renpy.game.contexts.append(self.context)
 
-        rng.pushback(self.random)
 
 
 class RollbackLog(renpy.object.Object):
@@ -859,7 +899,7 @@ class RollbackLog(renpy.object.Object):
     (weakref to object, information needed to rollback that object)
     """
 
-    __version__ = 3
+    __version__ = 4
 
 
     nosave = [ 'old_store', 'mutated' ]
@@ -883,6 +923,11 @@ class RollbackLog(renpy.object.Object):
         # Reset the RNG on the creation of a new game.
         rng.reset()
 
+        # True if we should retain data from here to the next checkpoint
+        # on load.
+        self.retain_after_load_flag = False
+
+
     def after_setstate(self):
         self.mutated = { }
         self.rolled_forward = False
@@ -893,6 +938,8 @@ class RollbackLog(renpy.object.Object):
         if version < 3:
             self.rollback_is_fixed = False
             self.fixed_rollback_boundary = None
+        if version < 4:
+            self.retain_after_load_flag = False
 
     def begin(self):
         """
@@ -926,6 +973,8 @@ class RollbackLog(renpy.object.Object):
                 self.rollback_is_fixed = False
 
         self.current = Rollback()
+        self.current.retain_after_load = self.retain_after_load_flag
+
         self.log.append(self.current)
 
         self.mutated = { }
@@ -1054,6 +1103,8 @@ class RollbackLog(renpy.object.Object):
         node.
         """
 
+        self.retain_after_load_flag = False
+
         if self.current.checkpoint:
             return
 
@@ -1100,6 +1151,15 @@ class RollbackLog(renpy.object.Object):
 
         self.rollback_limit = 0
 
+    def retain_after_load(self):
+        """
+        Called to return data from this statement until the next checkpoint
+        when the game is loaded.
+        """
+
+        self.retain_after_load_flag = True
+        self.current.retain_after_load = True
+
     def fix_rollback(self):
         if not self.rollback_is_fixed and len(self.log) > 1:
             self.fixed_rollback_boundary = self.log[-2].context.current
@@ -1111,7 +1171,7 @@ class RollbackLog(renpy.object.Object):
 
         return self.rollback_limit > 0
 
-    def rollback(self, checkpoints, force=False, label=None, greedy=True):
+    def rollback(self, checkpoints, force=False, label=None, greedy=True, on_load=False):
         """
         This rolls the system back to the first valid rollback point
         after having rolled back past the specified number of checkpoints.
@@ -1123,13 +1183,18 @@ class RollbackLog(renpy.object.Object):
         rolling back, otherwise if we run out of log this call has no
         effect.
 
-        @param label: The label that is called in the game script
-        after rollback has finished, if it exists.
+        `label`
+            A label that is called after rollback has finished, if the
+            label exists.
 
         `greedy`
             If true, rollback will keep going until just after the last
             checkpoint. If False, it will stop immediately before the
             current statement.
+
+        `on_load`
+            Should be true if this rollback is being called in response to a
+            load. Used to implement .retain_after_load()
         """
 
         # If we have exceeded the rollback limit, and don't have force,
@@ -1192,6 +1257,11 @@ class RollbackLog(renpy.object.Object):
             other_contexts = renpy.game.contexts[1:]
             renpy.game.contexts = renpy.game.contexts[0:1]
 
+        if on_load and revlog[-1].retain_after_load:
+            retained = revlog.pop()
+        else:
+            retained = None
+
         # Actually roll things back.
         for rb in revlog:
             rb.rollback()
@@ -1202,6 +1272,10 @@ class RollbackLog(renpy.object.Object):
             if rb.forward is not None:
                 self.forward.insert(0, (rb.context.current, rb.forward))
 
+        if retained is not None:
+            retained.rollback_control()
+            self.log.append(retained)
+
         # Disable the next transition, as it's pointless. (Only when not used with a label.)
         renpy.game.interface.suppress_transition = True
 
@@ -1289,7 +1363,7 @@ class RollbackLog(renpy.object.Object):
                 store[name] = value
 
         # Now, rollback to an acceptable point.
-        self.rollback(0, force=True, label=label, greedy=False)
+        self.rollback(0, force=True, label=label, greedy=False, on_load=True)
 
         # Because of the rollback, we never make it this far.
 
@@ -1342,6 +1416,24 @@ def py_eval(source, globals=None, locals=None): #@ReservedAssignment
     return eval(py_compile(source, 'eval'), globals, locals)
 
 
+def raise_at_location(e, loc):
+    """
+    Raises `e` (which must be an Exception object) at location `loc`.
+
+    `loc`
+        A location, which should be a (filename, line_number) tuple.
+    """
+
+    filename, line = loc
+
+    node = ast.parse("raise e", filename)
+    ast.increment_lineno(node, line - 1)
+    code = compile(node, filename, 'exec')
+
+    # PY3 - need to change to exec().
+    exec code in { "e" : e }
+
+
 # This was used to proxy accesses to the store. Now it's kept around to deal
 # with cases where it might have leaked into a pickle.
 class StoreProxy(object):
