@@ -1,4 +1,4 @@
-# Copyright 2004-2012 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -43,12 +43,12 @@ def increment_lineno(node, amount):
     for node in ast.walk(node):
         if hasattr(node, 'lineno'):
             node.lineno += amount
-    
+
 class LineNumberNormalizer(ast.NodeVisitor):
 
     def __init__(self):
         self.last_line = 1
-    
+
     def generic_visit(self, node):
 
         if hasattr(node, 'lineno'):
@@ -77,21 +77,33 @@ class Positional(object):
 
 # Used to generate the documentation
 all_keyword_names = set()
-            
+
 class Keyword(object):
     """
     This represents an optional keyword parameter to a function.
     """
-    
+
     def __init__(self, name):
         self.name = name
 
-        all_keyword_names.add(self.name) 
-       
+        all_keyword_names.add(self.name)
+
         if parser:
             parser.add(self)
 
-        
+STYLE_PREFIXES = [
+    '',
+    'insensitive_',
+    'hover_',
+    'idle_',
+    'activate_',
+    'selected_',
+    'selected_insensitive_',
+    'selected_hover_',
+    'selected_idle_',
+    'selected_activate_',
+]
+
 class Style(object):
     """
     This represents a style parameter to a function.
@@ -100,7 +112,7 @@ class Style(object):
     def __init__(self, name):
         self.name = name
 
-        for j in renpy.style.prefix_subs:
+        for j in STYLE_PREFIXES:
             all_keyword_names.add(j + self.name)
 
         if parser:
@@ -116,7 +128,7 @@ class PrefixStyle(object):
         self.prefix = prefix
         self.name = name
 
-        for j in renpy.style.prefix_subs:
+        for j in STYLE_PREFIXES:
             all_keyword_names.add(prefix + j + self.name)
 
         if parser:
@@ -129,7 +141,7 @@ class Parser(object):
 
         # The name of this object.
         self.name = name
-        
+
         # The positional arguments, keyword arguments, and child
         # statements of this statement.
         self.positional = [ ]
@@ -151,21 +163,21 @@ class Parser(object):
                 self.add(j)
 
             return
-        
+
         if isinstance(i, Positional):
             self.positional.append(i)
 
         elif isinstance(i, Keyword):
-            self.keyword[i.name] = i                
+            self.keyword[i.name] = i
 
         elif isinstance(i, Style):
-            for j in renpy.style.prefix_subs:
+            for j in STYLE_PREFIXES:
                 self.keyword[j + i.name] = i
 
         elif isinstance(i, PrefixStyle):
-            for j in renpy.style.prefix_subs:
+            for j in STYLE_PREFIXES:
                 self.keyword[i.prefix + j + i.name] = i
-                
+
         elif isinstance(i, Parser):
             self.children[i.name] = i
 
@@ -177,14 +189,14 @@ class Parser(object):
                 c = self.children[word].parse_layout(l, name)
             else:
                 c = self.children[word].parse(l, name)
-                
+
             return c
-        else:            
+        else:
             return None
 
     def parse_layout(self, l, name):
         l.error("The %s statement cannot be used as a container for the has statement." % self.name)
-        
+
     def parse_children(self, stmt, l, name):
         l.expect_block(stmt)
 
@@ -195,24 +207,24 @@ class Parser(object):
         with new_variable() as child_name:
 
             count = 0
-            
+
             while l.advance():
 
                 if len(l.block) != 1:
                     rv.extend(self.parse_exec("%s = (%s, %d)" % (child_name, name, count), l.number))
                 else:
                     child_name = name
-                    
+
                 c = self.parse_statement(l, child_name)
                 if c is None:
                     l.error('Expected screen language statement.')
 
                 rv.extend(c)
                 count += 1
-                    
+
 
         return rv
-    
+
     def parse_eval(self, expr, lineno=1):
         """
         Parses an expression for eval, and then strips off the module
@@ -221,17 +233,17 @@ class Parser(object):
 
         if isinstance(expr, unicode):
             expr = renpy.python.escape_unicode(expr)
-        
+
         try:
             rv = ast.parse(expr, 'eval').body[0].value
-        except SyntaxError, e:            
+        except SyntaxError, e:
             raise renpy.parser.ParseError(
                 filename,
                 lineno + e[1][1] - 1,
                 "Syntax error while parsing python expression.",
                 e[1][3],
                 e[1][2])
-            
+
         increment_lineno(rv, lineno-1)
 
         return rv
@@ -244,7 +256,7 @@ class Parser(object):
 
         if isinstance(code, unicode):
             code = renpy.python.escape_unicode(code)
-            
+
         try:
             rv = ast.parse(code, 'exec')
         except SyntaxError, e:
@@ -255,7 +267,7 @@ class Parser(object):
                 "Syntax error while parsing python code.",
                 e[1][3],
                 e[1][2])
-            
+
         increment_lineno(rv, lineno-1)
 
         return rv.body
@@ -277,13 +289,12 @@ class Parser(object):
         current statement.
         """
 
-        raise NotImplemented()
+        raise Exception("Not Implemented")
+
 
-    
-            
 # A singleton value.
-many = object()
-    
+many = renpy.object.Sentinel("many")
+
 class FunctionStatementParser(Parser):
     """
     This is responsible for parsing function statements.
@@ -292,13 +303,13 @@ class FunctionStatementParser(Parser):
     def __init__(self, name, function, nchildren=0, unevaluated=False, scope=False):
 
         super(FunctionStatementParser, self).__init__(name)
-        
+
         # Functions that are called when this statement runs.
         self.function = function
 
         # The number of children we have.
         self.nchildren = nchildren
-        
+
         # True if we should evaluate arguments and children. False
         # if we should just pass them into our child.
         self.unevaluated = unevaluated
@@ -314,7 +325,7 @@ class FunctionStatementParser(Parser):
 
     def parse_layout(self, l, name):
         return self.parse(l, name, True)
-            
+
     def parse(self, l, name, layout_mode=False):
 
         # The list of nodes this function returns.
@@ -322,15 +333,15 @@ class FunctionStatementParser(Parser):
 
         # The line number of the current node.
         lineno = l.number
-        
+
         if layout_mode and self.nchildren == 0:
             l.error("The %s statement cannot be used as a layout." % self.name)
-        
-        func = self.parse_eval(self.function, lineno)        
+
+        func = self.parse_eval(self.function, lineno)
 
         call_node = ast.Call(
             lineno=lineno,
-            col_offset=0, 
+            col_offset=0,
             func=func,
             args=[ ],
             keywords=[ ],
@@ -341,15 +352,15 @@ class FunctionStatementParser(Parser):
         seen_keywords = set()
 
         # Parses a keyword argument from the lexer.
-        def parse_keyword(l):
+        def parse_keyword(l, expect):
             name = l.word()
 
             if name is None:
-                l.error('expected a keyword argument, colon, or end of line.')
+                l.error(expect)
 
             if name not in self.keyword:
                 l.error('%r is not a keyword argument or valid child for the %s statement.' % (name, self.name))
-            
+
             if name in seen_keywords:
                 l.error('keyword argument %r appears more than once in a %s statement.' % (name, self.name))
 
@@ -360,7 +371,7 @@ class FunctionStatementParser(Parser):
             call_node.keywords.append(
                 ast.keyword(arg=str(name), value=expr),
                 )
-                
+
         # We assume that the initial keyword has been parsed already,
         # so we start with the positional arguments.
 
@@ -380,7 +391,7 @@ class FunctionStatementParser(Parser):
                 block = False
                 break
 
-            parse_keyword(l)
+            parse_keyword(l, "expected a keyword argument, colon, or end of line.")
 
         rv.append(ast.Expr(value=call_node))
 
@@ -388,13 +399,13 @@ class FunctionStatementParser(Parser):
             rv.extend(self.parse_exec('ui.child_or_fixed()'))
 
         needs_close = (self.nchildren != 0)
-                   
+
         # The index of the child we're adding to this statement.
         child_index = 0
 
         # A list of lexers we need to parse the contents of.
         lexers = [ ]
-        
+
         if block:
             lexers.append(l.subblock_lexer())
 
@@ -406,9 +417,9 @@ class FunctionStatementParser(Parser):
 
             # If we have a block, parse it. This also takes care of parsing the
             # block of a has clause.
-            
+
             for l in lexers:
-                
+
                 while l.advance():
 
                     state = l.checkpoint()
@@ -419,7 +430,7 @@ class FunctionStatementParser(Parser):
 
                         if child_index != 0:
                             l.error("The has statement may not be given after a child has been supplied.")
-                            
+
                         c = self.parse_statement(l, child_name, layout_mode=True)
 
                         if c is None:
@@ -427,14 +438,14 @@ class FunctionStatementParser(Parser):
 
                         # Remove the call to child_or_fixed.
                         rv.pop()
-                        
+
                         rv.extend(self.parse_exec("%s = (%s, %d)" % (child_name, name, child_index)))
                         rv.extend(c)
 
                         needs_close = False
-                        
+
                         continue
-                    
+
                     c = self.parse_statement(l, child_name)
 
                     if c is not None:
@@ -448,21 +459,24 @@ class FunctionStatementParser(Parser):
 
                     l.revert(state)
 
+                    if not l.eol():
+                        parse_keyword(l, "expected a keyword argument or child statement.")
+
                     while not l.eol():
-                        parse_keyword(l)
+                        parse_keyword(l, "expected a keyword argument or end of line.")
 
         if needs_close:
-            rv.extend(self.parse_exec("ui.close()"))        
+            rv.extend(self.parse_exec("ui.close()"))
 
         if "id" not in seen_keywords:
             call_node.keywords.append(ast.keyword(arg="id", value=self.parse_eval(name, lineno)))
-            
+
         if "scope" not in seen_keywords and self.scope:
             call_node.keywords.append(ast.keyword(arg="scope", value=self.parse_eval("_scope", lineno)))
-            
+
         return rv
 
-        
+
 ##############################################################################
 # Definitions of screen language statements.
 
@@ -494,7 +508,10 @@ position_property_names = [
         "yfill",
         # no center, since it can conflict with the center transform.
         "xcenter",
-        "ycenter",        
+        "ycenter",
+        "xsize",
+        "ysize",
+        "xysize",
         ]
 
 position_properties = [ Style(i) for i in position_property_names ]
@@ -503,6 +520,7 @@ side_position_properties = [ PrefixStyle("side_", i) for i in position_property_
 
 text_property_names = [
         "antialias",
+        "vertical",
         "black_color",
         "bold",
         "color",
@@ -522,7 +540,7 @@ text_property_names = [
         "minwidth",
         "min_width",
         "newline_indent",
-        "outlines",        
+        "outlines",
         "rest_indent",
         "ruby_style",
         "slow_cps",
@@ -564,7 +582,8 @@ window_properties = [ Style(i) for i in [
 button_properties = [ Style(i) for i in [
         "sound",
         "mouse",
-        "focus_mask",        
+        "focus_mask",
+        "child",
         ] ]
 
 bar_properties = [ Style(i) for i in [
@@ -589,6 +608,8 @@ bar_properties = [ Style(i) for i in [
 box_properties = [ Style(i) for i in [
         "box_layout",
         "box_wrap",
+        "box_reverse",
+        "order_reverse",
         "spacing",
         "first_spacing",
         "fit_first",
@@ -610,7 +631,7 @@ ui_properties = [
 def add(thing):
     parser.add(thing)
 
-    
+
 ##############################################################################
 # UI statements.
 
@@ -688,6 +709,7 @@ Keyword("exclude")
 Keyword("prefix")
 Keyword("suffix")
 Keyword("changed")
+Keyword("pixel_width")
 add(ui_properties)
 add(position_properties)
 add(text_properties)
@@ -702,6 +724,7 @@ Keyword("action")
 Keyword("clicked")
 Keyword("hovered")
 Keyword("unhovered")
+Keyword("alternate")
 add(ui_properties)
 add(position_properties)
 add(window_properties)
@@ -719,6 +742,7 @@ Keyword("action")
 Keyword("clicked")
 Keyword("hovered")
 Keyword("unhovered")
+Keyword("alternate")
 Keyword("image_style")
 add(ui_properties)
 add(position_properties)
@@ -731,6 +755,7 @@ Keyword("action")
 Keyword("clicked")
 Keyword("hovered")
 Keyword("unhovered")
+Keyword("alternate")
 Keyword("text_style")
 Keyword("substitute")
 Keyword("scope")
@@ -751,7 +776,7 @@ add(text_position_properties)
 add(text_text_properties)
 
 for name in [ "bar", "vbar" ]:
-    FunctionStatementParser(name, "ui." + name, 0)    
+    FunctionStatementParser(name, "ui." + name, 0)
     Keyword("adjustment")
     Keyword("range")
     Keyword("value")
@@ -761,13 +786,14 @@ for name in [ "bar", "vbar" ]:
     add(ui_properties)
     add(position_properties)
     add(bar_properties)
-    
+
 # Omit autobar. (behavior)
 
 FunctionStatementParser("viewport", "ui.viewport", 1)
 Keyword("child_size")
 Keyword("mousewheel")
 Keyword("draggable")
+Keyword("edgescroll")
 Keyword("xadjustment")
 Keyword("yadjustment")
 Keyword("xinitial")
@@ -888,8 +914,8 @@ class DefaultParser(Parser):
         code = "_scope.setdefault(%r, (%s))" % (name, rest)
 
         return self.parse_exec(code, l.number)
-        
-DefaultParser("default")        
+
+DefaultParser("default")
 
 
 class UseParser(Parser):
@@ -897,31 +923,35 @@ class UseParser(Parser):
     def __init__(self, name):
         super(UseParser, self).__init__(name)
         childbearing_statements.append(self)
-        
+
     def parse(self, l, name):
 
         lineno = l.number
-        
+
         target_name = l.require(l.word)
 
-        code = "renpy.use_screen(%r, _name=%s, _scope=_scope" % (target_name, name)
-        
+        code = "renpy.use_screen(%r" % target_name
+
         args = renpy.parser.parse_arguments(l)
 
         if args:
 
             for k, v in args.arguments:
                 if k is None:
-                    l.error('The use statement only takes keyword arguments.')
-                    
-                code += ", %s=(%s)" % (k, v)
-                    
+                    code += ", (%s)" % v
+                else:
+                    code += ", %s=(%s)" % (k, v)
+
+        code += ", _name=%s, _scope=_scope" % name
+
+        if args:
+
             if args.extrapos:
-                l.error('The use statement only takes keyword arguments.')
+                code += ", *(%s)" % args.extrapos
 
             if args.extrakw:
                 code += ", **(%s)" % args.extrakw
-                
+
         code += ")"
 
         return self.parse_exec(code, lineno)
@@ -936,33 +966,33 @@ class IfParser(Parser):
         childbearing_statements.append(self)
 
     def parse(self, l, name):
-        
+
         with new_variable() as child_name:
 
             count = 0
-            
+
             lineno = l.number
             condition = self.parse_eval(l.require(l.python_expression), lineno)
 
             l.require(':')
             l.expect_eol()
-            
+
             body = self.parse_exec("%s = (%s, %d)" % (child_name, name, count))
             body.extend(self.parse_children('if', l, child_name))
 
             orelse = [ ]
-            
+
             rv = ast.If(test=condition, body=body, orelse=orelse, lineno=lineno, col_offset=0)
 
             count += 1
-            
+
             state = l.checkpoint()
 
             while l.advance():
-                
+
                 old_orelse = orelse
                 lineno = l.number
-                
+
                 if l.keyword("elif"):
                     condition = self.parse_eval(l.require(l.python_expression), lineno)
 
@@ -982,18 +1012,18 @@ class IfParser(Parser):
                     old_orelse.extend(self.parse_children('if', l, child_name))
 
                     break
-                    
+
                 else:
                     l.revert(state)
                     break
-                
+
         return [ rv ]
 
 IfParser("if")
 
 
 class ForParser(Parser):
-        
+
     def __init__(self, name):
         super(ForParser, self).__init__(name)
         childbearing_statements.append(self)
@@ -1002,11 +1032,11 @@ class ForParser(Parser):
 
         is_tuple = False
         pattern = [ ]
-        
+
         while True:
 
             lineno = l.number
-                
+
             if l.match(r"\("):
                 p = self.parse_tuple_pattern(l)
             else:
@@ -1029,7 +1059,7 @@ class ForParser(Parser):
             return pattern[0]
         else:
             return ast.Tuple(elts=pattern, ctx=ast.Store())
-        
+
     def parse(self, l, name):
 
         lineno = l.number
@@ -1039,18 +1069,18 @@ class ForParser(Parser):
         l.require('in')
 
         expression = self.parse_eval(l.require(l.python_expression), l.number)
-                
+
         l.require(':')
         l.expect_eol()
 
         with new_variable() as counter_name:
-        
+
             with new_variable() as child_name:
 
                 children = self.parse_exec("%s = (%s, %s)" % (child_name, name, counter_name))
                 children.extend(self.parse_children('for', l, child_name))
                 children.extend(self.parse_exec("%s += 1" % counter_name))
-                
+
             rv = self.parse_exec("%s = 0" % counter_name)
 
             rv.append(ast.For(
@@ -1062,12 +1092,12 @@ class ForParser(Parser):
                     col_offset=0))
 
         return rv
-            
-ForParser("for")            
+
+ForParser("for")
 
 
 class PythonParser(Parser):
-        
+
     def __init__(self, name, one_line):
         super(PythonParser, self).__init__(name)
 
@@ -1085,7 +1115,7 @@ class PythonParser(Parser):
 
             python_code = l.python_block()
             lineno += 1
-            
+
         return self.parse_exec(python_code, lineno)
 
 PythonParser("$", True)
@@ -1110,7 +1140,7 @@ for i in childbearing_statements:
 
 #         for i, child in enumerate(self.children):
 #             child.evaluate(_name + (i,), _scope)
-    
+
 # def screen_function(positional, keyword, children):
 #     name = renpy.python.py_eval(positional[0].source)
 #     function = ScreenFunction(children)
@@ -1125,7 +1155,7 @@ for i in childbearing_statements:
 
 #     return values
 
-    
+
 # screen_stmt = FunctionStatementParser("screen", screen_function, unevaluated=True)
 # Positional("name", Word)
 # Keyword("modal", Expression)
@@ -1137,7 +1167,7 @@ class ScreenLangScreen(renpy.object.Object):
     """
     This represents a screen defined in the screen language.
     """
-    
+
     __version__ = 1
 
     variant = "None"
@@ -1145,7 +1175,9 @@ class ScreenLangScreen(renpy.object.Object):
     # Predict should be false for screens created before
     # prediction existed.
     predict = "False"
-    
+
+    parameters = None
+
     def __init__(self):
 
         # The name of the screen.
@@ -1159,7 +1191,7 @@ class ScreenLangScreen(renpy.object.Object):
 
         # The screen's tag.
         self.tag = None
-        
+
         # The PyCode object containing the screen's code.
         self.code = None
 
@@ -1168,15 +1200,18 @@ class ScreenLangScreen(renpy.object.Object):
 
         # Should we predict this screen?
         self.predict = "None" # expr.
-        
+
+        # The parameters this screen takes.
+        self.parameters = None
+
     def after_upgrade(self, version):
         if version < 1:
             self.modal = "False"
             self.zorder = "0"
-        
+
     def define(self):
         """
-        Defines us as a screen.
+        Defines a screen.
         """
 
         renpy.display.screen.define_screen(
@@ -1186,18 +1221,30 @@ class ScreenLangScreen(renpy.object.Object):
             zorder=self.zorder,
             tag=self.tag,
             variant=renpy.python.py_eval(self.variant),
-            predict=renpy.python.py_eval(self.predict)
+            predict=renpy.python.py_eval(self.predict),
+            parameters=self.parameters,
             )
 
-    def __call__(self, _scope=None, **kwargs):
-        renpy.python.py_exec_bytecode(self.code.bytecode, locals=_scope)
-        
+    def __call__(self, *args, **kwargs):
+        scope = kwargs["_scope"]
+
+        if self.parameters:
+
+            args = scope.get("_args", ())
+            kwargs = scope.get("_kwargs", { })
+
+            values = renpy.ast.apply_arguments(self.parameters, args, kwargs)
+            scope.update(values)
+
+        renpy.python.py_exec_bytecode(self.code.bytecode, locals=scope)
+
+
 
 class ScreenParser(Parser):
 
     def __init__(self):
         super(ScreenParser, self).__init__("screen")
-        
+
     def parse(self, l, name="_name"):
 
         location = l.get_location()
@@ -1219,30 +1266,30 @@ class ScreenParser(Parser):
             if l.match('variant'):
                 screen.variant = l.require(l.simple_expression)
                 return True
-                
+
             if l.match('predict'):
                 screen.predict = l.require(l.simple_expression)
                 return True
-                
+
             return False
 
-        
         lineno = l.number
-        
+
         screen.name = l.require(l.word)
+        screen.parameters = renpy.parser.parse_parameters(l)
 
         while parse_keyword(l):
             continue
 
         l.require(':')
-        l.expect_eol()        
+        l.expect_eol()
         l.expect_block('screen statement')
 
         l = l.subblock_lexer()
 
         rv = [ ]
         count = 0
-        
+
         with new_variable() as child_name:
 
             while l.advance():
@@ -1260,31 +1307,31 @@ class ScreenParser(Parser):
 
                 if c is None:
                     l.error('Expected a screen language statement.')
-                
+
                 rv.extend(c)
                 count += 1
-                
+
         node = ast.Module(body=rv, lineno=lineno, col_offset=0)
         ast.fix_missing_locations(node)
         LineNumberNormalizer().visit(node)
 
         # Various bits of debugging code:
-        
+
         # print ast.dump(node, True, True)
 
         # a = compile(node, 'foo', 'exec')
         # import dis
         # dis.dis(a)
-        
-        # import unparse
-        # print
-        # print screen.name, "-----------------------------------------"
-        # unparse.Unparser(node)
-        
+
+#        import unparse
+#        print
+#        print screen.name, "-----------------------------------------"
+#        unparse.Unparser(node)
+
         screen.code = renpy.ast.PyCode(node, location, 'exec')
 
         return screen
-                
+
 screen_parser = ScreenParser()
 screen_parser.add(all_statements)
 
@@ -1296,6 +1343,6 @@ def parse_screen(l):
     global filename
 
     filename = l.filename
-    
+
     screen = screen_parser.parse(l)
     return screen
