@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -142,3 +142,7 @@ def call(method, parsed, *args, **kwargs):
         return None
 
     return method(parsed, *args, **kwargs)
+
+def get_name(parsed):
+    name, _parsed = parsed
+    return " ".join(name)
