@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -57,6 +57,10 @@ class ScriptTranslator(object):
         # A map from language to the StringTranslator for that language.
         self.strings = collections.defaultdict(StringTranslator)
 
+        # A map from language to a list of TranslateBlock objects for
+        # that language.
+        self.block = collections.defaultdict(list)
+
         # A map from language to a list of TranslatePython objects for
         # that language.
         self.python = collections.defaultdict(list)
@@ -86,6 +90,9 @@ class ScriptTranslator(object):
             if isinstance(n, renpy.ast.TranslatePython):
                 self.python[n.language].append(n)
 
+            if isinstance(n, renpy.ast.TranslateBlock):
+                self.block[n.language].append(n)
+
             elif isinstance(n, renpy.ast.Menu):
 
                 for i in n.items:
@@ -436,7 +443,7 @@ def init_translation():
     """
 
     global style_backup
-    style_backup = renpy.style.backup()
+    style_backup = renpy.style.backup() # @UndefinedVariable
 
     load_all_rpts()
 
@@ -452,8 +459,14 @@ def change_language(language):
 
     tl = renpy.game.script.translator
 
-    renpy.style.restore(style_backup)
-    renpy.style.rebuild()
+    renpy.style.restore(style_backup) # @UndefinedVariable
+    renpy.style.rebuild() # @UndefinedVariable
+
+    def run_blocks():
+        for i in tl.block[language]:
+            renpy.game.context().run(i.block[0])
+
+    renpy.game.invoke_in_new_context(run_blocks)
 
     for i in tl.python[language]:
         renpy.python.py_exec_bytecode(i.code.bytecode)
@@ -466,7 +479,7 @@ def change_language(language):
     renpy.exports.free_memory()
 
     # Rebuild the styles.
-    renpy.style.rebuild()
+    renpy.style.rebuild() # @UndefinedVariable
 
     # Restart the interaction.
     renpy.exports.restart_interaction()
