@@ -1,4 +1,4 @@
-# Copyright 2004-2013 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2014 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -192,6 +192,33 @@ class ChildOrFixed(Addable):
         if d is not None:
             raise Exception("Did not expect to close %r." % d)
 
+class ChildList(Addable):
+    """
+    Used by SL2 to interface with SL1 screens. This appends widgets to
+    a list of children.
+    """
+
+    def __init__(self, children, style_prefix):
+        self.children = children
+        self.style_prefix = style_prefix
+
+    def get_style_group(self):
+        style_prefix = self.style_prefix
+
+        if style_prefix:
+            return style_prefix[:-1]
+        else:
+            return None
+
+    style_group = property(get_style_group)
+
+    def add(self, d, key):
+        self.children.append(d)
+
+    def close(self, d):
+        raise Exception("Spurious ui.close().")
+
+
 # A stack of things we can add to.
 stack = [ ]
 
@@ -331,7 +358,7 @@ def context_enter(w):
 def context_exit(w):
     close(w)
 
-NoStyleGroupGiven = object()
+NoStyleGroupGiven = renpy.object.Sentinel("NoStyleGroupGiven")
 
 def style_group_style(s, style_group):
     """
@@ -343,17 +370,11 @@ def style_group_style(s, style_group):
         style_group = stack[-1].style_group
 
     if style_group is None:
-        return s
+        new_style = s
+    else:
+        new_style = style_group + "_" + s
 
-    new_style = style_group + "_" + s
-
-    if new_style[0] == "_":
-        return new_style
-
-    if new_style not in renpy.style.style_map:
-        renpy.style.style_map[new_style] = renpy.style.Style(s, heavy=False, name=new_style)
-
-    return new_style
+    return renpy.style.get_style(new_style) # @UndefinedVariable
 
 # The screen we're using as we add widgets. None if there isn't a
 # screen.
@@ -773,7 +794,7 @@ def imagemap_compat(ground,
                     hotspots,
                     unselected=None,
                     style='imagemap',
-                    button_style='imagemap_button',
+                    button_style='hotspot',
                     **properties):
 
     if isinstance(button_style, basestring):
@@ -830,7 +851,7 @@ def _imagebutton(idle_image = None,
                  auto=None,
                  **properties):
 
-    def choice(a, b, name):
+    def choice(a, b, name, required=False):
         if a:
             return a
 
@@ -840,9 +861,12 @@ def _imagebutton(idle_image = None,
         if auto is not None:
             return renpy.config.imagemap_auto_function(auto, name)
 
+        if required:
+            raise Exception("Could not find a %s image for imagemap." % name)
+
         return None
 
-    idle = choice(idle, idle_image, "idle")
+    idle = choice(idle, idle_image, "idle", required=True)
     hover = choice(hover, hover_image, "hover")
     insensitive = choice(insensitive, insensitive_image, "insensitive")
     selected_idle = choice(selected_idle, selected_idle_image, "selected_idle")
@@ -862,27 +886,7 @@ def _imagebutton(idle_image = None,
 
 imagebutton = Wrapper(_imagebutton, style="image_button")
 
-def get_text_style(style, default):
-    if isinstance(style, basestring):
-        base = style
-        rest = ()
-    else:
-        base = style.name[0]
-        rest = style.name[1:]
-
-    base = base + "_text"
-
-    rv = renpy.style.style_map.get(base, None)
-
-    if rv is None:
-        rv = renpy.style.style_map[default]
-
-    for i in rest:
-        rv = rv[i]
-
-    return rv
-
-def textbutton(label, clicked=None, style=None, text_style=None, substitute=True, scope=None, **kwargs):
+def _textbutton(label, clicked=None, style=None, text_style=None, substitute=True, scope=None, **kwargs):
 
     button_kwargs = { }
     text_kwargs = { }
@@ -904,12 +908,16 @@ def textbutton(label, clicked=None, style=None, text_style=None, substitute=True
         style = style_group_style('button', NoStyleGroupGiven)
 
     if text_style is None:
-        text_style = get_text_style(style, style_group_style('button_text', NoStyleGroupGiven))
+        text_style = renpy.style.get_text_style(style, style_group_style('button_text', NoStyleGroupGiven)) # @UndefinedVariable
 
-    button(style=style, clicked=clicked, **button_kwargs)
-    text(label, style=text_style, substitute=substitute, scope=scope, **text_kwargs)
+    rv = renpy.display.behavior.Button(style=style, clicked=clicked, **button_kwargs)
+    rv.add(renpy.text.text.Text(label, style=text_style, substitute=substitute, scope=scope, **text_kwargs))
+    return rv
 
-def label(label, style=None, text_style=None, substitute=True, scope=None, **kwargs):
+def textbutton(label, **kwargs):
+    return add(_textbutton(label, **kwargs))
+
+def _label(label, style=None, text_style=None, substitute=True, scope=None, **kwargs):
 
     label_kwargs = { }
     text_kwargs = { }
@@ -924,10 +932,14 @@ def label(label, style=None, text_style=None, substitute=True, scope=None, **kwa
         style = style_group_style('label', NoStyleGroupGiven)
 
     if text_style is None:
-        text_style = get_text_style(style, style_group_style('label_text', NoStyleGroupGiven))
+        text_style = renpy.style.get_text_style(style, style_group_style('label_text', NoStyleGroupGiven)) # @UndefinedVariable
 
-    window(style=style, **label_kwargs)
-    text(label, style=text_style, substitute=substitute, scope=scope, **text_kwargs)
+    rv = renpy.display.layout.Window(None, style=style, **label_kwargs)
+    rv.add(renpy.text.text.Text(label, style=text_style, substitute=substitute, scope=scope, **text_kwargs))
+    return rv
+
+def label(label, **kwargs):
+    return add(_label(label, **kwargs))
 
 adjustment = renpy.display.behavior.Adjustment
 
@@ -1130,7 +1142,7 @@ def _imagemap(ground=None, hover=None, insensitive=None, idle=None, selected_hov
 
 imagemap = Wrapper(_imagemap, imagemap=True, style='imagemap')
 
-def _hotspot(spot, style='imagemap_button', **properties):
+def _hotspot(spot, style='hotspot', **properties):
 
     if not imagemap_stack:
         raise Exception("hotspot expects an imagemap to be defined.")
@@ -1278,11 +1290,7 @@ def on(event, action=[], id=None): #@ReservedAssignment
     if renpy.display.screen.current_screen().current_transform_event != event:
         return
 
-    if isinstance(action, (list, tuple)):
-        for i in action:
-            i()
-    else:
-        action()
+    renpy.display.behavior.run(action)
 
 
 ##############################################################################
